@@ -1,6 +1,6 @@
 ;; mazewar.scm --- Server-side implementation of gzochi mazewar example game
 
-;; Copyright (C) 2012 Julian Graham
+;; Copyright (C) 2014 Julian Graham
 ;;
 ;; This software is provided 'as-is', without any express or implied
 ;; warranty. In no event will the authors be held liable for any damages
@@ -264,11 +264,7 @@
        ;; 200 milliseconds in the future.
 
        (else (mazewar:set-missile-space! missile next-tile)
-	     (gzochi:schedule-task 
-	      (gzochi:make-callback 'move-missile 
-				    '(gzochi example mazewar) 
-				    missile) 
-	      200)))))
+	     (gzochi:schedule-task (g:@ move-missile missile) 200)))))
 
   ;; Send `player-hidden' messages to the specified player for each of the
   ;; players visible from the player's current location. To be called right 
@@ -433,9 +429,7 @@
      ((eqv? cmd mazewar:message-type-client-shoot)
       (let ((missile (mazewar:make-missile orientation p space)))
 	(mazewar:set-player-missile! p missile)
-	(gzochi:schedule-task (gzochi:make-callback 
-			       'move-missile '(gzochi example mazewar) missile)
-			      200)))))
+	(gzochi:schedule-task (g:@ move-missile missile) 200)))))
 
   ;; The `logged-in' callback. Called when a new player attempts to join the
   ;; game.
@@ -501,6 +495,5 @@
       ;; indicating a successful login.
       
       (gzochi:make-client-session-listener
-       (gzochi:make-callback 'received-message '(gzochi example mazewar) player)
-       (gzochi:make-callback 'disconnected '(gzochi example mazewar) player))))
+       (g:@ received-message player) (g:@ disconnected player))))
 )
