@@ -1,5 +1,5 @@
 /* protocol.c: Client-side protocol I/O routines for libgzochi
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -25,13 +25,19 @@
 
 #define CHUNK_SIZE 512
 
+#if defined (__APPLE__) && defined (__MACH__)
+#define SEND_FLAGS 0x00
+#else
+#define SEND_FLAGS MSG_NOSIGNAL
+#endif
+
 static int send_fully (int sock, unsigned char *data, int len)
 {
   int total_sent = 0;
   while (total_sent < len)
-    {
-      int bytes_sent = send 
-	(sock, (char *) data + total_sent, len - total_sent, MSG_NOSIGNAL);
+    {      
+      int bytes_sent = send
+	(sock, (char *) data + total_sent, len - total_sent, SEND_FLAGS);
       
       if (bytes_sent < 0)
 	return -1;
@@ -100,8 +106,14 @@ static void dispatch_session_message
 static void dispatch_session_disconnected 
 (gzochi_client_common_session *session)
 {
-  if (session->disconnected_callback != NULL)
-    session->disconnected_callback (session, session->disconnected_user_data);
+  session->connected = FALSE;
+  if (! session->disconnect_acknowledged)
+    {
+      session->disconnect_acknowledged = TRUE;
+      if (session->disconnected_callback != NULL)
+	session->disconnected_callback 
+	  (session, session->disconnected_user_data);
+    }
 }
 
 static void dispatch 
@@ -131,6 +143,12 @@ static int attempt_dispatch (gzochi_client_common_session *session, int limit)
       short message_len = 0;
       char opcode = 0x00;
 
+      if (! session->connected)
+	{
+	  dispatch_session_disconnected (session);
+	  break;
+	}
+
       if (session->buffer_length - offset < 3)
 	break;
 
@@ -175,7 +193,7 @@ int gzochi_client_protocol_read (gzochi_client_common_session *session)
   int bytes_requested = MIN(CHUNK_SIZE, available_buffer);
   int bytes_read = recv (session->socket, chunk, bytes_requested, 0);
 
-  if (bytes_read < 0)
+  if (bytes_read <= 0)
     return -1;
       
   memcpy (session->buffer + session->buffer_length, chunk, bytes_read);
