@@ -1,5 +1,5 @@
 /* client.c: Main client API routines for libgzochi
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ static int read_and_dispatch (gzochi_client_session *session, int connecting)
   
       if (gzochi_client_protocol_read (session) < 0)
 	{
+	  session->connected = FALSE;
 	  gzochi_client_common_session_disconnect (session);
 	  return -1;
 	}
@@ -95,6 +96,9 @@ gzochi_client_session *gzochi_client_connect
     return NULL;
 
   setsockopt (sock, IPPROTO_TCP, TCP_NODELAY, (char *) &flag, sizeof (int));
+#if defined (__APPLE__) && defined (__MACH__)
+  setsockopt (sock, SOL_SOCKET, SO_NOSIGPIPE, (char *) &flag, sizeof (int));
+#endif
 
   session = gzochi_client_common_session_new ();
   session->connected = TRUE;
