@@ -1,5 +1,5 @@
 /* client.c: GSource interface for libgzochi-glib
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -65,8 +65,9 @@ static gboolean check (GSource *source)
 
   if (poll_fd->revents & G_IO_IN)
     {
-      gzochi_client_protocol_read (gzochi_source->session);
-      return gzochi_client_common_session_is_dispatchable 
+      if (gzochi_client_protocol_read (gzochi_source->session) < 0)
+	gzochi_source->session->connected = FALSE;
+      return gzochi_client_common_session_is_dispatchable
 	(gzochi_source->session);
     }
   if (poll_fd->revents & G_IO_HUP || poll_fd->revents & G_IO_ERR)
@@ -117,6 +118,7 @@ read_and_dispatch (gzochi_glib_client_session *session, int connecting)
   
       if (gzochi_client_protocol_read (session) < 0)
 	{
+	  session->connected = FALSE;
 	  if (session->disconnected_callback != NULL)
 	    session->disconnected_callback 
 	      (session, session->disconnected_user_data);
@@ -154,6 +156,9 @@ gzochi_glib_client_session *gzochi_glib_client_connect
     return NULL;
 
   setsockopt (sock, IPPROTO_TCP, TCP_NODELAY, (char *) &flag, sizeof (int));
+#if defined (__APPLE__) && defined (__MACH__)
+  setsockopt (sock, SOL_SOCKET, SO_NOSIGPIPE, (char *) &flag, sizeof (int));
+#endif
 
   session = gzochi_client_common_session_new ();
   session->connected = TRUE;
