@@ -1,5 +1,5 @@
 /* channel.c: Channel management routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -239,7 +239,7 @@ static void close_channel
   iter = g_sequence_get_begin_iter (channel->sessions);
   tx_context = join_side_effects_transaction (context, channel);
 
-  g_mutex_lock (context->client_mapping_lock);
+  g_mutex_lock (&context->client_mapping_lock);
 
   while (!g_sequence_iter_is_end (iter))
     {
@@ -310,7 +310,7 @@ static void close_channel
       iter = g_sequence_iter_next (iter);
     }
         
-  g_mutex_unlock (context->client_mapping_lock);
+  g_mutex_unlock (&context->client_mapping_lock);
   mpz_clear (session_oid);
   free (channel_oid_str);
 }
@@ -345,7 +345,7 @@ static void send_channel_message
   tx_context = join_side_effects_transaction (context, channel);
   iter = g_sequence_get_begin_iter (channel->sessions);
 
-  g_mutex_lock (context->client_mapping_lock);
+  g_mutex_lock (&context->client_mapping_lock);
 
   while (!g_sequence_iter_is_end (iter))
     {
@@ -354,28 +354,33 @@ static void send_channel_message
 	g_hash_table_lookup (context->oids_to_clients, session_oid_str);
 
       if (client != NULL)
-	tx_context->messages = g_list_append 
-	  (tx_context->messages, 
-	   gzochid_channel_payload_message_new 
-	   (GZOCHID_CHANNEL_MESSAGE_SEND, client, send_op->message, 
-	    send_op->len));
+	{
+	  tx_context->messages = g_list_append 
+	    (tx_context->messages, 
+	     gzochid_channel_payload_message_new 
+	     (GZOCHID_CHANNEL_MESSAGE_SEND, client, send_op->message, 
+	      send_op->len));
+	  iter = g_sequence_iter_next (iter);
+	}
       else 
 	{
+	  GSequenceIter *old_iter = iter;
+
+	  iter = g_sequence_iter_next (iter);
+
 	  gzochid_warning 
 	    ("Client not found for messaged channel session '%s'; removing.", 
 	     session_oid_str);
-	  g_sequence_remove (iter);
+	  g_sequence_remove (old_iter);
 	  channel_modified = TRUE;
 	}
-
-      iter = g_sequence_iter_next (iter);
     }
 
   if (channel_modified)
     gzochid_data_mark 
       (context, &gzochid_channel_serialization, channel, NULL);
   
-  g_mutex_unlock (context->client_mapping_lock);
+  g_mutex_unlock (&context->client_mapping_lock);
 }
 
 static void join_channel
@@ -731,7 +736,7 @@ static gpointer deserialize_channel
   channel->id = gzochid_util_deserialize_bytes (in, (int *) &channel->id_len);
 
   channel->sessions = gzochid_util_deserialize_sequence 
-    (in, (gpointer (*) (GString *)) gzochid_util_deserialize_string);
+    (in, (gpointer (*) (GString *)) gzochid_util_deserialize_string, free);
 
   gzochid_util_deserialize_mpz (in, channel->oid);
   gzochid_util_deserialize_mpz (in, channel->scm_oid);
@@ -778,7 +783,7 @@ gzochid_channel *gzochid_channel_new (char *name)
   gzochid_channel *channel = calloc (1, sizeof (gzochid_channel));
 
   channel->name = name;
-  channel->sessions = g_sequence_new (NULL);
+  channel->sessions = g_sequence_new (free);
   mpz_init (channel->oid);
   mpz_init (channel->scm_oid);
 
