@@ -1,5 +1,5 @@
 /* config.c: Configuration management routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -64,14 +64,17 @@ long gzochid_config_to_long (char *str, long def)
 GHashTable *gzochid_config_keyfile_extract_config 
 (GKeyFile *key_file, char *group)
 {
-  unsigned int i = 0, num_keys = 0;
+  unsigned int i = 0;
+  gsize num_keys = 0;
   char **keys = g_key_file_get_keys (key_file, group, &num_keys, NULL);
-  GHashTable *config = g_hash_table_new (g_str_hash, g_str_equal);
+  GHashTable *config = g_hash_table_new_full 
+    (g_str_hash, g_str_equal, free, free);
 
   for (; i < num_keys; i++) 
     g_hash_table_insert 
       (config, keys[i], g_key_file_get_value (key_file, group, keys[i], NULL));
   
+  free (keys);
   return config;
 }
 
