@@ -1,5 +1,5 @@
 /* data.c: Application data management routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -68,7 +68,8 @@ static gzochid_data_transaction_context *create_transaction_context
   else tx_context->transaction = gzochid_storage_transaction_begin 
 	 (app_context->storage_context);
 
-  tx_context->oids_to_references = g_hash_table_new (g_str_hash, g_str_equal);
+  tx_context->oids_to_references = g_hash_table_new_full 
+    (g_str_hash, g_str_equal, free, NULL);
   tx_context->ptrs_to_references = g_hash_table_new 
     (g_direct_hash, g_direct_equal);
 
@@ -82,6 +83,8 @@ static void free_oid_block (void *ptr)
   mpz_clear (block->first);
   mpz_clear (block->next);
   mpz_clear (block->last);
+
+  free (block);
 } 
 
 static void transaction_context_free (gzochid_data_transaction_context *context)
@@ -127,13 +130,15 @@ static gboolean flush_reference
 
       if (gzochid_transaction_rollback_only ())
 	{
-	  g_string_free (out, FALSE);	  
+	  free (oid_str);
+	  g_string_free (out, TRUE);	  
 	  return FALSE;
 	}
 
       gzochid_storage_transaction_put
 	(context->transaction, context->context->oids, oid_str, 
 	 strlen (oid_str) + 1, out->str, out->len);
+      free (oid_str);
 
       event = malloc (sizeof (gzochid_application_data_event));
       base_event = (gzochid_application_event *) event;
@@ -145,7 +150,7 @@ static gboolean flush_reference
       gzochid_application_event_dispatch
 	(context->context->event_source, base_event);
 
-      g_string_free (out, FALSE);
+      g_string_free (out, TRUE);
 
       break;
     case GZOCHID_MANAGED_REFERENCE_STATE_REMOVED_FETCHED:
@@ -190,6 +195,15 @@ static int data_prepare (gpointer data)
   return TRUE;
 }
 
+static void free_reference (gpointer data)
+{
+  gzochid_data_managed_reference *reference = 
+    (gzochid_data_managed_reference *) data;
+
+  mpz_clear (reference->oid);
+  free (data);
+}
+
 static void finalize_references (gzochid_data_transaction_context *context)
 {
   GList *references = g_hash_table_get_values (context->oids_to_references);
@@ -208,7 +222,7 @@ static void finalize_references (gzochid_data_transaction_context *context)
 	}
       reference_ptr = reference_ptr->next;
     }
-  g_list_free (references);
+  g_list_free_full (references, free_reference);
 }
 
 static void data_commit (gpointer data)
@@ -277,6 +291,7 @@ static gzochid_data_oid_block *create_oid_block
   next_oid_value_len = strlen (next_oid_value) + 1;
   gzochid_storage_put 
     (context->meta, next_oid_key, 1, next_oid_value, next_oid_value_len);
+  free (next_oid_value);
 
   mpz_sub_ui (block->last, block->last, 1);
   
@@ -290,7 +305,7 @@ static gzochid_data_oid_block *reserve_oids
 {
   gzochid_data_oid_block *block = NULL;
   
-  g_mutex_lock (context->free_oids_lock);
+  g_mutex_lock (&context->free_oids_lock);
   
   if (g_list_length (context->free_oid_blocks) > 0)
     {
@@ -300,7 +315,7 @@ static gzochid_data_oid_block *reserve_oids
     }
   else block = create_oid_block (context);
 
-  g_mutex_unlock (context->free_oids_lock);
+  g_mutex_unlock (&context->free_oids_lock);
   return block;
 }
 
@@ -509,6 +524,8 @@ static void dereference
 	(context->context->event_source, base_event);
 
       in = g_string_new_len (data, data_len);
+      free (data);
+
       reference->obj = reference->serialization->deserializer 
 	(context->context, in);
       reference->state = GZOCHID_MANAGED_REFERENCE_STATE_NOT_MODIFIED;
