@@ -1,5 +1,5 @@
 /* game.c: Game context management routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -63,11 +63,13 @@ static void scan_app_dir (gzochid_game_context *context, char *dir)
     {
       gzochid_warning 
 	("%s does not exist or is not a regular file.", descriptor_file);
+      g_free (descriptor_file);
       return;
     }
 
   descriptor = gzochid_config_parse_application_descriptor (descriptor_file);
   initialize_application (context, descriptor);
+  g_free (descriptor_file);    
 }
 
 static void scan_apps_dir (gzochid_game_context *context)
@@ -211,8 +213,8 @@ void gzochid_game_context_init
 
   context->port = gzochid_config_to_int
     (g_hash_table_lookup (config, "server.port"), 0);
-  context->apps_dir = g_hash_table_lookup (config, "server.fs.apps");
-  context->work_dir = g_hash_table_lookup (config, "server.fs.data");
+  context->apps_dir = strdup (g_hash_table_lookup (config, "server.fs.apps"));
+  context->work_dir = strdup (g_hash_table_lookup (config, "server.fs.data"));
 
   tx_timeout_ms = gzochid_config_to_long 
     (g_hash_table_lookup (config, "tx.timeout"), DEFAULT_TX_TIMEOUT_MS);
