@@ -1,5 +1,5 @@
 /* gzochid.c: Main server bootstrapping routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -111,6 +111,12 @@ static void initialize_async (gpointer data, gpointer user_data)
   gzochid_context_until 
     (server_context->game_context, GZOCHID_GAME_STATE_RUNNING);
 
+  g_hash_table_unref (admin_config);
+  g_hash_table_unref (game_config);
+  g_hash_table_unref (log_config);
+
+  g_key_file_free (key_file);
+
   gzochid_fsm_to_state (context->fsm, GZOCHID_STATE_RUNNING);
 }
 
@@ -231,8 +237,6 @@ int main (int argc, char *argv[])
 
   context = gzochid_server_context_new ();
 
-  g_thread_init (NULL);
-
   gzochid_server_context_init (context);
   gzochid_fsm_until (((gzochid_context *) context)->fsm, GZOCHID_STATE_STOPPED);
   gzochid_server_context_free (context);
