@@ -1,5 +1,5 @@
 /* scheme.c: Supplementary Scheme routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -29,6 +29,7 @@
 #include "data.h"
 #include "guile.h"
 #include "io.h"
+#include "log.h"
 #include "reloc.h"
 #include "scheme.h"
 #include "session.h"
@@ -80,6 +81,7 @@ static SCM scm_make_object_removed_condition;
 static SCM scm_make_name_exists_condition;
 static SCM scm_make_name_not_bound_condition;
 static SCM scm_transaction_retry_condition_p;
+static SCM scm_transaction_aborted_condition_p;
 
 static SCM resolve_procedure (char *procedure, GList *module)
 {
@@ -142,6 +144,16 @@ static gboolean is_transaction_retry (SCM cond)
   return scm_is_true (scm_call_1 (scm_transaction_retry_condition_p, cond));
 }
 
+static gboolean is_transaction_aborted (SCM cond)
+{
+  return scm_is_true (scm_call_1 (scm_transaction_aborted_condition_p, cond));
+}
+
+static gboolean triggered_by_rollback (SCM cond)
+{
+  return is_transaction_aborted (cond) && gzochid_transaction_rollback_only ();
+}
+
 void gzochid_scheme_application_worker 
 (gzochid_application_context *context, gzochid_auth_identity *identity, 
  gpointer ptr)
@@ -158,9 +170,10 @@ void gzochid_scheme_application_worker
   *ret = scheme_invoke (context, identity, proc, args, exception_var);
 
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    gzochid_transaction_mark_for_rollback 
-      (&scheme_participant, 
-       is_transaction_retry (scm_variable_ref (exception_var)));
+    if (! triggered_by_rollback (scm_variable_ref (exception_var)))
+      gzochid_transaction_mark_for_rollback 
+	(&scheme_participant, 
+	 is_transaction_retry (scm_variable_ref (exception_var)));
 }
 
 void gzochid_scheme_application_task_worker 
@@ -168,6 +181,8 @@ void gzochid_scheme_application_task_worker
  gpointer task)
 {
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+  GList *args = g_list_append
+    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "task");
 
   gzochid_transaction_join (&scheme_participant, NULL);
 
@@ -175,16 +190,17 @@ void gzochid_scheme_application_task_worker
     (context, 
      identity,
      "gzochi:run-task", 
-     g_list_append
-     (g_list_append 
-      (g_list_append (NULL, "gzochi"), "private"), "task"), 
+     args,
      scm_list_1 ((SCM) task), 
      exception_var);
 
+  g_list_free (args);
+
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    gzochid_transaction_mark_for_rollback 
-      (&scheme_participant,
-       is_transaction_retry (scm_variable_ref (exception_var)));
+    if (! triggered_by_rollback (scm_variable_ref (exception_var)))
+      gzochid_transaction_mark_for_rollback 
+	(&scheme_participant,
+	 is_transaction_retry (scm_variable_ref (exception_var)));
 }
 
 static gzochid_application_callback *scm_to_callback 
@@ -213,6 +229,8 @@ void gzochid_scheme_application_initialized_worker
   SCM callback = gzochid_scheme_create_callback 
     (context->descriptor->initialized, NULL);
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
+  GList *args = g_list_append
+    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
 
   callback_reference = gzochid_data_create_reference 
     (context, &gzochid_scheme_data_serialization, callback);
@@ -223,9 +241,7 @@ void gzochid_scheme_application_initialized_worker
     (context,
      identity,
      "gzochi:execute-initialized",
-     g_list_append
-     (g_list_append
-      (g_list_append (NULL, "gzochi"), "private"), "app"),
+     args,
      scm_list_2 (callback,
 		 gzochid_scheme_ghashtable_to_hashtable 
 		 (properties, 
@@ -235,10 +251,15 @@ void gzochid_scheme_application_initialized_worker
 		  (SCM (*) (gpointer)) scm_from_locale_string)),
      exception_var);
 
+  g_list_free (args);
+
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    gzochid_transaction_mark_for_rollback 
-      (&scheme_participant,
-       is_transaction_retry (scm_variable_ref (exception_var)));
+    {
+      if (! triggered_by_rollback (scm_variable_ref (exception_var)))
+	gzochid_transaction_mark_for_rollback 
+	  (&scheme_participant,
+	   is_transaction_retry (scm_variable_ref (exception_var)));
+    }
   else gzochid_data_set_binding_to_oid 
 	 (context, "s.initializer", callback_reference->oid, NULL);
 }
@@ -281,7 +302,8 @@ void gzochid_scheme_application_logged_in_worker
 
   SCM handler = SCM_BOOL_F;
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
-
+  GList *args = NULL;
+    
   mpz_init (session_oid);
   mpz_set_str (session_oid, session_oid_str, 16);
   session_reference = gzochid_data_create_reference_to_oid
@@ -296,6 +318,9 @@ void gzochid_scheme_application_logged_in_worker
       return;
     }
 
+  args = g_list_append
+    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
+
   session = (gzochid_client_session *) session_reference->obj;
 
   scm_session = gzochid_scheme_create_client_session 
@@ -311,16 +336,19 @@ void gzochid_scheme_application_logged_in_worker
     (context,
      identity,
      "gzochi:execute-logged-in",
-     g_list_append
-     (g_list_append
-      (g_list_append (NULL, "gzochi"), "private"), "app"),
+     args,
      scm_list_2 ((SCM) callback_reference->obj, (SCM) scm_session), 
      exception_var);
   
+  g_list_free (args);
+
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    gzochid_transaction_mark_for_rollback 
-      (&scheme_participant, 
-       is_transaction_retry (scm_variable_ref (exception_var)));
+    {
+      if (! triggered_by_rollback (scm_variable_ref (exception_var)))
+	gzochid_transaction_mark_for_rollback 
+	  (&scheme_participant, 
+	   is_transaction_retry (scm_variable_ref (exception_var)));
+    }
   else if (scm_is_false (handler))
     {
       gzochid_client_session_send_login_failure (context, session);
@@ -378,6 +406,8 @@ void gzochid_scheme_application_received_message_worker
   session_reference = gzochid_data_create_reference_to_oid 
     (context, &gzochid_client_session_serialization, session_oid);
 
+  mpz_clear (session_oid);
+
   gzochid_data_dereference (session_reference, &err);
 
   if (err != NULL)
@@ -411,10 +441,13 @@ void gzochid_scheme_application_received_message_worker
 
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
     {
-      gzochid_transaction_join (&scheme_participant, NULL);
-      gzochid_transaction_mark_for_rollback 
-	(&scheme_participant, 
-	 is_transaction_retry (scm_variable_ref (exception_var)));
+      if (! triggered_by_rollback (scm_variable_ref (exception_var)))
+	{
+	  gzochid_transaction_join (&scheme_participant, NULL);
+	  gzochid_transaction_mark_for_rollback 
+	    (&scheme_participant, 
+	     is_transaction_retry (scm_variable_ref (exception_var)));
+	}
     }
 
   g_list_free (gpa);
@@ -430,12 +463,12 @@ void gzochid_scheme_application_disconnected_worker
   gzochid_data_managed_reference *callback_reference = NULL;
 
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
-  GList *gpa = g_list_append
-    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
-
+  GList *gpa = NULL;
+  char *oid_str = (char *) ptr;
   mpz_t session_oid;
+
   mpz_init (session_oid);
-  mpz_set_str (session_oid, (char *) ptr, 16);
+  mpz_set_str (session_oid, oid_str, 16);
 
   session_reference = gzochid_data_create_reference_to_oid 
     (context, &gzochid_client_session_serialization, session_oid);
@@ -445,6 +478,13 @@ void gzochid_scheme_application_disconnected_worker
 
   if (err != NULL)
     {
+      if (err->message != NULL)
+	gzochid_warning
+	  ("Failed to dereference disconnecting session '%s': %s", 
+	   oid_str, err->message);
+      else gzochid_warning 
+	     ("Failed to dereference disconnecting session '%s'.", oid_str);
+
       g_error_free (err);
       return;
     }
@@ -461,7 +501,7 @@ void gzochid_scheme_application_disconnected_worker
 
       g_list_free (gpa);
 
-      gzochid_data_remove_object (session_reference, NULL);
+      gzochid_client_session_disconnected_worker (context, identity, oid_str);
       return;
     }
 
@@ -474,10 +514,22 @@ void gzochid_scheme_application_disconnected_worker
   
   if (err != NULL)
     {
-      g_error_free (err);
+      if (err->message != NULL)
+	gzochid_warning
+	  ("Failed to retrieve disconnect callback for session '%s': %s", 
+	   oid_str, err->message);
+      else gzochid_warning 
+	     ("Failed to retrieve disconnect callback for session '%s'.", 
+	      oid_str);
+
+      g_error_free (err);      
+      gzochid_client_session_disconnected_worker (context, identity, oid_str);
       return;
     }
-  
+
+  gpa = g_list_append
+    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "app");
+
   gzochid_scheme_invoke
     (context, 
      identity,
@@ -487,12 +539,16 @@ void gzochid_scheme_application_disconnected_worker
 
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
     {
-      gzochid_transaction_join (&scheme_participant, NULL);
-      gzochid_transaction_mark_for_rollback 
-	(&scheme_participant, 
-	 is_transaction_retry (scm_variable_ref (exception_var)));
+      if (! triggered_by_rollback (scm_variable_ref (exception_var)))
+	{
+	  gzochid_transaction_join (&scheme_participant, NULL);
+	  gzochid_transaction_mark_for_rollback 
+	    (&scheme_participant, 
+	     is_transaction_retry (scm_variable_ref (exception_var)));
+	}
     }
-  else gzochid_data_remove_object (session_reference, NULL);
+
+  gzochid_client_session_disconnected_worker (context, identity, oid_str);
   g_list_free (gpa);
 }
 
@@ -578,7 +634,9 @@ static void *scheme_managed_record_deserializer
   int vec_len = gzochi_common_io_read_int ((unsigned char *) in->str, 0);
   SCM vec = SCM_EOL, port = SCM_EOL, record = SCM_EOL;
   SCM exception_var = scm_make_variable (SCM_UNSPECIFIED);
-  
+  GList *args = g_list_append 
+    (g_list_append (g_list_append (NULL, "gzochi"), "private"), "data");
+     
   g_string_erase (in, 0, 4);
   vec = scm_take_u8vector ((unsigned char *) in->str, vec_len);
 
@@ -588,21 +646,23 @@ static void *scheme_managed_record_deserializer
     (context, 
      NULL,
      "gzochi:deserialize-managed-record", 
-     g_list_append 
-     (g_list_append 
-      (g_list_append (NULL, "gzochi"), "private"), "data"),
+     args,
      scm_list_1 (port),
      exception_var);
 
+  g_list_free (args);
   g_string_erase (in, 0, vec_len);
   
   /* Don't join the transaction here as the serializer should only be
      called during the PREPARING phase, which is too late to join. */
   
   if (scm_variable_ref (exception_var) != SCM_UNSPECIFIED)
-    gzochid_transaction_mark_for_rollback 
-      (&scheme_participant, 
-       is_transaction_retry (scm_variable_ref (exception_var)));
+    {
+      if (! triggered_by_rollback (scm_variable_ref (exception_var))) 
+	gzochid_transaction_mark_for_rollback 
+	  (&scheme_participant, 
+	   is_transaction_retry (scm_variable_ref (exception_var)));
+    }
   else scm_gc_protect_object (record);
 
   return record;
@@ -991,6 +1051,8 @@ static void *initialize_bindings (void *ptr)
 	    "gzochi:make-object-removed-condition");
   bind_scm ("gzochi conditions", &scm_transaction_retry_condition_p,
 	    "gzochi:transaction-retry-condition?");
+  bind_scm ("gzochi conditions", &scm_transaction_aborted_condition_p,
+	    "gzochi:transaction-aborted-condition?");
 
   bind_scm ("gzochi data", &scm_make_managed_hashtable, 
 	    "gzochi:make-managed-hashtable");
