@@ -1,5 +1,5 @@
 /* session.c: Client session management routines for gzochid
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -145,9 +145,20 @@ static int session_prepare (gpointer data)
   return TRUE;
 }
 
+static void free_operation (gpointer data)
+{
+  gzochid_client_session_pending_operation *op = 
+    (gzochid_client_session_pending_operation *) data;
+
+  mpz_clear (op->target_session);
+  free (op);
+}
+
 static void cleanup_transaction 
 (gzochid_client_session_transaction_context *tx_context)
 {
+  g_list_free_full (tx_context->operations, free_operation);
+  
   free (tx_context);
 }
 
@@ -161,7 +172,7 @@ static void session_commit_operation
   char *oid_str = mpz_get_str (NULL, 16, op->target_session);
   gzochid_protocol_client *client = NULL;
 
-  g_mutex_lock (context->client_mapping_lock);
+  g_mutex_lock (&context->client_mapping_lock);
   client = (gzochid_protocol_client *) g_hash_table_lookup 
     (context->oids_to_clients, oid_str);
 
@@ -169,7 +180,7 @@ static void session_commit_operation
     {
       gzochid_warning 
 	("Client not found for session '%s'; skipping operation.", oid_str);
-      g_mutex_unlock (context->client_mapping_lock);
+      g_mutex_unlock (&context->client_mapping_lock);
       free (oid_str);
       return;
     }
@@ -200,7 +211,7 @@ static void session_commit_operation
       break;
     }
 
-  g_mutex_unlock (context->client_mapping_lock);
+  g_mutex_unlock (&context->client_mapping_lock);
 }
 
 static void session_commit_operation_visitor (gpointer data, gpointer user_data)
@@ -255,7 +266,7 @@ static gpointer deserialize_client_session
   gzochid_auth_identity *identity = 
     gzochid_auth_identity_deserializer (context, in);
   GSequence *channels = gzochid_util_deserialize_sequence 
-    (in, (gpointer (*) (GString *)) gzochid_util_deserialize_string);
+    (in, (gpointer (*) (GString *)) gzochid_util_deserialize_string, free);
   gzochid_client_session *session = gzochid_client_session_new (identity);
 
   g_sequence_free (session->channels);
@@ -304,6 +315,7 @@ static void finalize_handler
     (context, handler->received_message);
   gzochid_application_callback_serialization.finalizer 
     (context, handler->disconnected);
+  free (handler);
 }
 
 static void finalize_client_session
@@ -362,7 +374,21 @@ static void remove_session
 	  
 	  gzochid_data_remove_object (scm_session_reference, &local_err);
 
-	  if (local_err == NULL)
+	  if (local_err != NULL)
+	    {
+	      /* This might happen if the disconnect callback (or some other
+		 piece of code) has already deleted the SCM representation of
+		 the session, or if the login handler never completed
+		 successfully. */
+
+	      if (local_err->message != NULL)
+		gzochid_info 
+		  ("Unable to remove Scheme object for session '%s': %s",
+		   local_err->message);
+
+	      g_clear_error (&local_err);
+	    }
+	    
 	    gzochid_data_remove_object (session_reference, &local_err);
 	  
 	  if (local_err != NULL)
@@ -373,7 +399,7 @@ static void remove_session
     }
   else g_propagate_error (err, local_err);
    
-  g_string_free (binding, FALSE);
+  g_string_free (binding, TRUE);
 }
 
 void gzochid_client_session_disconnected_worker
@@ -568,7 +594,7 @@ static void prefix_binding_persistence_task_worker
   gzochid_data_set_binding_to_oid 
     (context, binding->str, persistence_task->holder->oid, NULL);
 
-  g_string_free (binding, FALSE);
+  g_string_free (binding, TRUE);
 }
 
 gzochid_task *gzochid_data_prefix_binding_persistence_task_new
@@ -630,7 +656,8 @@ void gzochid_client_session_persist
   gzochid_data_prefix_binding_persistence_task_free (task);
 }
 
-void gzochid_sweep_client_sessions (gzochid_application_context *context)
+void gzochid_sweep_client_sessions 
+(gzochid_application_context *context, gzochid_auth_identity *identity)
 {
   mpz_t oid;
   GError *err = NULL;
@@ -650,17 +677,10 @@ void gzochid_sweep_client_sessions (gzochid_application_context *context)
       char *oid_str = mpz_get_str (NULL, 16, oid);
       char *next_next_binding = NULL;
       
-      remove_session (context, oid_str, &err);
+      gzochid_scheme_application_disconnected_worker 
+	(context, identity, oid_str);      
 
-      free (oid_str);
-
-      assert (err == NULL || err->code == GZOCHID_DATA_ERROR_NOT_FOUND);
-
-      if (err != NULL)
-	{
-	  g_error_free (err);
-	  err = NULL;
-	}
+      assert (! gzochid_transaction_rollback_only ());
       
       next_next_binding = gzochid_data_next_binding_oid 
 	(context, next_binding, oid, &err);
