@@ -1,5 +1,5 @@
 /* session.h: Prototypes and declarations for session.c
- * Copyright (C) 2013 Julian Graham
+ * Copyright (C) 2014 Julian Graham
  *
  * gzochi is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -62,6 +62,7 @@ void gzochid_client_session_send_message
 
 void gzochid_client_session_persist 
 (gzochid_application_context *, gzochid_client_session *, mpz_t);
-void gzochid_sweep_client_sessions (gzochid_application_context *);
+void gzochid_sweep_client_sessions 
+(gzochid_application_context *, gzochid_auth_identity *);
 
 #endif /* GZOCHID_SESSION_H */
