@@ -20,6 +20,7 @@
 #include <libserveez.h>
 #include <signal.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "context.h"
 #include "fsm.h"
@@ -44,13 +45,28 @@ static int detect_proto (svz_server_t *server, svz_socket_t *sock)
   return 1;
 }
 
-static int disconnected_socket (svz_socket_t *sock)
+static int write_socket (svz_socket_t *sock)
 {
   gzochid_socket_server_context *context = 
     (gzochid_socket_server_context *) sock->data;
   gzochid_protocol_client *client = 
     g_hash_table_lookup (context->clients, sock);
-  
+  int ret = 0;
+
+  g_mutex_lock (&client->sock_mutex);
+  ret = client->write_socket (sock);
+  g_mutex_unlock (&client->sock_mutex);
+
+  return ret;
+}
+
+static int disconnected_socket (svz_socket_t *sock)
+{  
+  gzochid_socket_server_context *context = 
+    (gzochid_socket_server_context *) sock->data;
+  gzochid_protocol_client *client = 
+    g_hash_table_lookup (context->clients, sock);
+
   gzochid_debug ("Socket disconnected.");
   gzochid_protocol_client_disconnected (client);
 
@@ -99,16 +115,19 @@ static int connect_socket (svz_server_t *server, svz_socket_t *sock)
 {
   gzochid_socket_server_context *context = 
     (gzochid_socket_server_context *) server->data;
+  gzochid_protocol_client *client = gzochid_protocol_client_accept (sock);
 
   sock->data = context;
   sock->disconnected_socket = disconnected_socket;
   sock->check_request = check_request;
   sock->handle_request = server->handle_request;
+  sock->write_socket = write_socket;
+
+  svz_tcp_nodelay (sock->sock_desc, 1, NULL);
   
   gzochid_debug ("Received socket connection."); 
 
-  g_hash_table_insert
-    (context->clients, sock, gzochid_protocol_client_accept (sock));
+  g_hash_table_insert (context->clients, sock, client);
   return 0;
 }
 
@@ -178,6 +197,12 @@ static void initialize_async (gpointer data, gpointer user_data)
   gzochid_fsm_to_state (context->fsm, GZOCHID_SOCKET_SERVER_STATE_RUNNING);
 }
 
+static void sigint_proxy (int s)
+{
+  signal (s, SIG_DFL);
+  raise (s);
+}
+
 static void initialize (int from_state, int to_state, gpointer user_data)
 {
   gzochid_context *context = (gzochid_context *) user_data;
@@ -188,7 +213,10 @@ static void initialize (int from_state, int to_state, gpointer user_data)
   svz_portcfg_t *portcfg = NULL;
   svz_server_t *server = NULL;
   
-  struct sigaction orig_act;
+  struct sigaction sigint_handler;
+
+  memset (&sigint_handler, 0, sizeof (struct sigaction));
+  sigint_handler.sa_handler = sigint_proxy;
 
   svz_boot ("gzochid");
 
@@ -213,7 +241,6 @@ static void initialize (int from_state, int to_state, gpointer user_data)
   server = svz_server_get ("game-default");
   server->data = server_context;
 
-  sigaction (SIGINT, NULL, &orig_act);
   svz_updn_all_coservers (-1);
 
   if (svz_updn_all_servers (1) < 0)
@@ -222,7 +249,7 @@ static void initialize (int from_state, int to_state, gpointer user_data)
   if (svz_server_bind (server, portcfg) != 0)
     gzochid_err ("Failed to bind server to port %d", game_context->port);
 
-  sigaction (SIGINT, &orig_act, NULL);
+  sigaction (SIGINT, &sigint_handler, NULL);
   gzochid_notice ("Game server listening on port %d", game_context->port);
 
   gzochid_thread_pool_push 
